from typing import Callable, Sequence, Any

from approval_utilities.utilities.exceptions.multiple_exceptions import (
    MultipleExceptions,
)


class ExceptionCollector:
    def __init__(self):
        self._exceptions = []

    def gather(self, code_to_execute: Callable):
        try:
            code_to_execute()
        except Exception as exception:
            self._exceptions.append(exception)

    def release(self):
        if len(self._exceptions) == 0:
            return
        if len(self._exceptions) == 1:
            raise self._exceptions[0]

        raise MultipleExceptions(self._exceptions)


def gather_all_exceptions(
    parameters: Sequence[Any], code_to_execute: Callable[[Any], Any]
) -> ExceptionCollector:
    collector = ExceptionCollector()
    for parameter in parameters:
        collector.gather(lambda p=parameter: code_to_execute(p))

    return collector


def gather_all_exceptions_and_throw(
    parameters: Sequence[Any], code_to_execute: Callable[[Any], Any]
) -> None:
    gather_all_exceptions(parameters, code_to_execute).release()



import os
import unittest

from approvaltests.namer.default_namer_factory import get_default_namer, NamerFactory
from approvaltests.namer.stack_frame_namer import StackFrameNamer


class NamerTests(unittest.TestCase):
    def test_class(self):
        self.assertEqual("NamerTests", StackFrameNamer().get_class_name())

    def test_method(self):
        n = StackFrameNamer()
        self.assertEqual("test_method", n.get_method_name())

    def test_name_works_from_inside_an_other_method(self):
        self.an_other_method()

    def an_other_method(self):
        n = StackFrameNamer()
        self.assertEqual(
            "test_name_works_from_inside_an_other_method", n.get_method_name()
        )

    def test_file(self):
        directory = StackFrameNamer().get_directory()
        assert os.path.exists(directory + "/test_namer.py")

    def test_basename(self):
        n = StackFrameNamer()
        self.assertTrue(
            n.get_basename().endswith("NamerTests.test_basename"), n.get_basename()
        )

    def test_received_name(self):
        filename = StackFrameNamer().get_received_filename("./stuff")
        self.assertEqual(filename, "./stuff.received.txt")

    def test_approved_name(self):
        filename = StackFrameNamer().get_approved_filename("./stuff")
        self.assertEqual(filename, "./stuff.approved.txt")

    def test_alternative_extension(self):
        n = StackFrameNamer(extension=".html")
        filename = n.get_approved_filename("./stuff")
        self.assertEqual(filename, "./stuff.approved.html")

    def test_additional_information(self):
        assert_ends_with(
            NamerFactory.with_parameters("case1").namer.get_approved_filename(),
            "test_additional_information.case1.approved.txt",
        )
        assert_ends_with(
            NamerFactory.with_parameters(
                "case1", "case2"
            ).namer.get_approved_filename(),
            "test_additional_information.case1.case2.approved.txt",
        )
        assert_ends_with(
            get_default_namer().get_approved_filename(),
            "test_additional_information.approved.txt",
        )


def assert_ends_with(received_string, expected_ending):
    assert received_string.endswith(expected_ending)


if __name__ == "__main__":
    unittest.main()



import os

from typing_extensions import ContextManager


def use_utc_timezone() -> ContextManager:
    class TimeZoneSwap:
        def __init__(self):
            self.timezone = ""

        def __enter__(self):
            self.timezone = os.environ.get("TZ")
            os.environ["TZ"] = "UCT"

        def __exit__(self, exc_type, exc_val, exc_tb):
            if self.timezone is None:
                os.environ.pop("TZ")
            else:
                os.environ["TZ"] = self.timezone

    return TimeZoneSwap()



from inspect import FrameInfo
from pathlib import Path
from typing import Callable, Any

from approval_utilities.utilities.clipboard_utilities import copy_to_clipboard
from approval_utilities.utilities.multiline_string_utils import remove_indentation_from
from approvaltests import (
    StackFrameNamer,
    assert_equal_with_reporter,
    Options,
    Reporter,
    verify,
    verify_all,
)
from approvaltests.inline.parse_docstring import parse_docstring
from approvaltests.reporters import MultiReporter


def get_approved_via_doc_string():
    test_stack_frame: FrameInfo = StackFrameNamer.get_test_frame()
    method: Callable[..., Any] = get_caller_method(test_stack_frame)
    return remove_indentation_from(method.__doc__)


def get_caller_method(caller_frame) -> Callable:
    caller_function_name: str = caller_frame[3]
    caller_function_object = caller_frame.frame.f_globals.get(
        caller_function_name, None
    )
    return caller_function_object


# Todo:
# detect the actual tab
# detect if the quote type used in the docstring  (i.e. " or ')


def fizz_buzz(param):
    return_string = ""
    for i in range(1, param + 1):
        if i % 15 == 0:
            return_string += "FizzBuzz\n"
        elif i % 3 == 0:
            return_string += "Fizz\n"
        elif i % 5 == 0:
            return_string += "Buzz\n"
        else:
            return_string += str(i) + "\n"
    return return_string


def test_fizz_buzz():
    """
    1
    2
    Fizz
    4
    Buzz
    Fizz
    7
    8
    """
    verify(fizz_buzz(8), options=Options().inline())


def test_docstrings():
    """
    hello
    world
    """
    # verify_inline(greetting())
    # verify(greetting(), options=Options().inline(show_code= False))
    verify(greeting(), options=Options().inline(show_code=True))


def greeting():
    return "hello\nworld"


class InlineReporter(Reporter):
    # TODO: Start here - Make this report create a temp file of the fixed source,
    #  and compare it with the existing source.
    # if there are mulitple failures, they each get there on reporter
    def report(self, received_path: str, approved_path: str) -> bool:
        received = Path(received_path).read_text()
        copy_to_clipboard(f"'''\n{received}\n'''")


def verify_inline(actual):
    options = Options()
    options = options.with_reporter(MultiReporter(options.reporter, InlineReporter()))
    if actual[-1] != "\n":
        actual += "\n"
    assert_equal_with_reporter(get_approved_via_doc_string(), actual, options=options)


def test_docstring_parsing():
    """
    1
    2 -> 2
    Fizz
    4
    Buzz
    Fizz
    7
    8
    """
    verify_all("inputs", parse_docstring())


def test_uppercase():
    """
    a -> A
    b -> B
    c -> C
    """
    verify(
        "\n".join([f"{a} -> {a.upper()}" for a in parse_docstring()]),
        options=Options().inline(),
    )



class StringWrapper:
    def __init__(self):
        self.string = ""

    def append(self, text):
        self.string += text

    def __str__(self):
        return self.string

    def __repr__(self):
        return self.string



import pytest
from approvaltests import verify, get_scenario_namer
from approvaltests.namer import NamerFactory
from approval_utilities.utilities.exceptions.exception_collector import (
    gather_all_exceptions_and_throw,
)


def is_leap_year(year: int) -> bool:
    if year % 400 == 0:
        return True
    if year % 100 == 0:
        return False
    return year % 4 == 0


@pytest.mark.parametrize("year", [1993, 1992, 1900, 2000])
def test_scenarios_old(year: int) -> None:
    verify(
        f"is Leap {str(year)}: {str(is_leap_year(year))}",
        namer=get_scenario_namer(year),
    )


# begin-snippet: parametrized-test-example
@pytest.mark.parametrize("year", [1993, 1992, 1900, 2000])
def test_scenarios(year: int) -> None:
    verify(
        f"is Leap {str(year)}: {str(is_leap_year(year))}",
        options=NamerFactory.with_parameters(year),
    )


# end-snippet


def test_manual_scenarios_with_blocking() -> None:
    # begin-snippet: multiple-verifies-with-blocking
    year = 1992
    verify(
        f"is Leap {str(year)}: {str(is_leap_year(year))}",
        options=NamerFactory.with_parameters(year),
    )
    year = 1993
    verify(
        f"is Leap {str(year)}: {str(is_leap_year(year))}",
        options=NamerFactory.with_parameters(year),
    )
    year = 1900
    verify(
        f"is Leap {str(year)}: {str(is_leap_year(year))}",
        options=NamerFactory.with_parameters(year),
    )
    year = 2000
    verify(
        f"is Leap {str(year)}: {str(is_leap_year(year))}",
        options=NamerFactory.with_parameters(year),
    )
    # end-snippet


def test_manual_scenarios() -> None:
    # begin-snippet: multiple-verifies-without-blocking
    years = [1993, 1992, 1900, 2000]
    gather_all_exceptions_and_throw(
        years,
        lambda y: verify(
            f"is Leap {str(y)}: {str(is_leap_year(y))}",
            options=NamerFactory.with_parameters(y),
        ),
    )
    # end-snippet
